/****************************************************************************
**  LinkTest                                                               **
*****************************************************************************
**  Copyright (c) 2008-2022                                                **
**  Forschungszentrum Juelich, Juelich Supercomputing Centre               **
**                                                                         **
**  See the file COPYRIGHT in the package base directory for details       **
****************************************************************************/
#ifndef LINKTEST_ERROR_H
#define LINKTEST_ERROR_H

#define REPORT_NONE 0
#define REPORT_FATAL 1
#define REPORT_ERROR 2
#define REPORT_WARN 3
#define REPORT_INFO 4
#define REPORT_DEBUG 5

#ifndef REPORT_LEVEL
#define REPORT_LEVEL REPORT_WARN
#endif

constexpr int SUCCESS = 0;
constexpr int ERROR   = 1;

void linktest_fatal(const char* file, const char* func, long line, const char* fmt, ...);
void linktest_error(const char* file, const char* func, long line, const char* fmt, ...);
void linktest_warn(const char* file, const char* func, long line, const char* fmt, ...);
void linktest_info(const char* file, const char* func, long line, const char* fmt, ...);
void linktest_debug(const char* file, const char* func, long line, const char* fmt, ...);

/* Error handling macros. fatal prints an error message and the terminates the
 * execution.
 *
 * The names are pretty generic so we have to be careful to avoid naming conflicts
 * that result in hard to understand compiler errors.
 */
// NOLINTBEGIN
#define fatal(fmt, ...) linktest_fatal(__FILE__, __func__, __LINE__, fmt, ## __VA_ARGS__)
#define error(fmt, ...) linktest_error(__FILE__, __func__, __LINE__, fmt, ## __VA_ARGS__)
#define warn(fmt, ...) linktest_warn(__FILE__, __func__, __LINE__, fmt, ## __VA_ARGS__)
#define info(fmt, ...) linktest_info(__FILE__, __func__, __LINE__, fmt, ## __VA_ARGS__)
#define debug(fmt, ...) linktest_debug(__FILE__, __func__, __LINE__, fmt, ## __VA_ARGS__)
// NOLINTEND
#endif